"""
Django settings for example project.

Generated by 'django-admin startproject' using Django 1.11.22.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""

import environs

# Read env variables
# https://github.com/sloria/environs
env = environs.Env()


# Build paths inside the project like this: BASE_DIR / '...'
BASE_DIR = env.path('BASE_DIR', default=str(environs.Path(__file__).parents[1]))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = env.str('SECRET_KEY', default='_(=en)od($b50-4)pqfe+2vh-vbql!2kpyc-29%yw-x@5br*5l')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = env.bool('DEBUG', default=True)

ALLOWED_HOSTS = env.list('ALLOWED_HOSTS', default=[])


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'example.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'example.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.11/ref/settings/#databases

DATABASES = {'default': env.dj_db_url('DATABASE_URL', default='sqlite:///' + str(BASE_DIR / 'db.sqlite3'))}


# Password validation
# https://docs.djangoproject.com/en/1.11/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/1.11/topics/i18n/

LANGUAGE_CODE = env.str('LANGUAGE_CODE', default='en-us')

TIME_ZONE = env.str('TIME_ZONE', default='UTC')

USE_I18N = env.bool('USE_I18N', default=True)

USE_L10N = env.bool('USE_L10N', default=True)

USE_TZ = env.bool('USE_TZ', default=True)


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.11/howto/static-files/

STATIC_URL = env.str('STATIC_URL', default='/static/')



import re
from typing import Iterable, NamedTuple, Optional

from django.template import Engine
from redbaron import RedBaron

DJANGO_SETTINGS_FRAGMENT = "Django settings"


class Replacement(NamedTuple):
    name: str
    env_type: str
    default: Optional[str] = None
    value_template: str = "{value}"
    env_variable: Optional[str] = None


REPLACEMENTS = tuple(
    Replacement(*rep)
    for rep in (
        ("BASE_DIR", "path", "str(environs.Path(__file__).parents[1])"),
        ("SECRET_KEY", "str"),
        ("DEBUG", "bool"),
        ("ALLOWED_HOSTS", "list"),
        (
            "DATABASES",
            "dj_db_url",
            "'sqlite:///' + str(BASE_DIR / 'db.sqlite3')",
            "{{'default': {value}}}",
            "DATABASE_URL",
        ),
        ("LANGUAGE_CODE", "str"),
        ("TIME_ZONE", "str"),
        ("USE_I18N", "bool"),
        ("USE_L10N", "bool"),
        ("USE_TZ", "bool"),
        ("STATIC_URL", "str"),
    )
)


class RedBaronSettings(RedBaron):
    def fix(self, replacements: Iterable[Replacement]):
        self.add_env()
        self.replace_settings(replacements)
        self.replace_comment()

    def add_env(self):
        # replace os import with environs
        import_node = self.find("ImportNode")
        import_node.value = "environs"
        # initialize env
        start = import_node.index_on_parent + 1
        lines = (
            "\n",
            "# Read env variables",
            "# https://github.com/sloria/environs",
            "env = environs.Env()",
            "\n",
        )
        for idx, line in zip(range(start, start + len(lines)), lines):
            self.insert(idx, line)

    def replace_settings(self, replacements: Iterable[Replacement]):
        for rep in replacements:
            node = self.find("name", rep.name)

            if rep.default is None:
                # get default value from template
                default = node.parent.value.dumps()
            else:
                default = rep.default

            value = "env.{env_type}('{env_variable}', default={default})".format(
                env_type=rep.env_type,
                env_variable=rep.env_variable or rep.name,
                default=default,
            )
            node.parent.value = rep.value_template.format(value=value)

    def replace_comment(self):
        to_replace = "os.path.join(BASE_DIR, ...)"
        compiled = re.compile(".*{to_replace}".format(to_replace=re.escape(to_replace)))
        comment = self.find("CommentNode", value=compiled)
        comment.value = comment.value.replace(to_replace, "BASE_DIR / '...'")


class EnvironsEngine(Engine):
    def from_string(self, template_code):
        if DJANGO_SETTINGS_FRAGMENT in template_code:
            red = RedBaronSettings(template_code)
            red.fix(REPLACEMENTS)
            template_code = red.dumps()

        return super().from_string(template_code)



"""
Invokes django-environs when the django_environs module is run as a script.
Example: python -m django_environs startproject
"""
from django_environs import management

if __name__ == "__main__":
    management.execute_from_command_line()



"""
This file is meant to be copied into project created with `django-environs startproject`
and run with env variables from tox.ini
"""
import os
from pathlib import Path

from django.conf import settings
from django.test import TestCase

ENV_SETTINGS = (
    ("BASE_DIR", Path("/")),
    ("SECRET_KEY", "secret"),
    ("DEBUG", False),
    ("ALLOWED_HOSTS", ["test", "testserver"]),
    ("LANGUAGE_CODE", "en-gb"),
    ("TIME_ZONE", "Africa/Abidjan"),
    ("USE_I18N", False),
    ("USE_L10N", False),
    ("USE_TZ", False),
    ("STATIC_URL", "/test/"),
)


class SettingsTestCase(TestCase):
    def test_settings(self):
        for name, value in ENV_SETTINGS:
            self.assertEqual(getattr(settings, name), value)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['django_environs', 'django_environs.management']

package_data = \
{'': ['*'], 'django_environs.management': ['commands/*']}

install_requires = \
['django', 'environs[django]', 'redbaron']

extras_require = \
{'dev': ['black', 'isort'], 'test': ['tox']}

entry_points = \
{'console_scripts': ['django-environs = '
                     'django_environs.management:execute_from_command_line']}

setup(name='django-environs',
      version='0.1.1',
      description='A cli tool for bootstraping django projects with environs settings',
      author='Jarosław Wygoda',
      author_email='jaroslaw@wygoda.me',
      url='https://github.com/jwygoda/django-environs',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )



import functools
from unittest.mock import patch

from django.core.management import (
    execute_from_command_line as django_execute_from_command_line,
)
from django.core.management import find_commands

from django_environs.template import EnvironsEngine


@functools.lru_cache(maxsize=None)
def get_commands():
    commands = {name: "django_environs" for name in find_commands(__path__[0])}
    return commands


@patch("django.core.management.get_commands", new=get_commands)
@patch("django.template.Engine", new=EnvironsEngine)
def execute_from_command_line(**kwargs):
    """Run a ManagementUtility."""
    django_execute_from_command_line(**kwargs)
